@@ -245,7 +245,7 @@ namespace libtorrent
 				t->get_policy().peer_is_interesting(*this);
 		}
 		// may throw an asio error if socket has disconnected
-		catch (std::exception& e) {}
+		catch (std::exception&) {}
 
 		TORRENT_ASSERT(is_interesting() == interested);
 	}
@@ -1107,7 +1107,7 @@ namespace libtorrent
 				"s: " << r.start << " | "
 				"l: " << r.length << " | "
 				"i: " << m_peer_interested << " | "
-				"t: " << (int)t->torrent_file().piece_size(r.piece) << " | "
+				"t: " << t->torrent_file().piece_size(r.piece) << " | "
 				"n: " << t->torrent_file().num_pieces() << " ]\n";
 #endif
 			write_reject_request(r);
@@ -1127,7 +1127,7 @@ namespace libtorrent
 				"s: " << r.start << " | "
 				"l: " << r.length << " | "
 				"i: " << m_peer_interested << " | "
-				"t: " << (int)t->torrent_file().piece_size(r.piece) << " | "
+				"t: " << t->torrent_file().piece_size(r.piece) << " | "
 				"n: " << t->torrent_file().num_pieces() << " ]\n";
 
 			(*m_logger) << time_now_string()
@@ -1194,7 +1194,7 @@ namespace libtorrent
 				"s: " << r.start << " | "
 				"l: " << r.length << " | "
 				"i: " << m_peer_interested << " | "
-				"t: " << (int)t->torrent_file().piece_size(r.piece) << " | "
+				"t: " << t->torrent_file().piece_size(r.piece) << " | "
 				"n: " << t->torrent_file().num_pieces() << " | "
 				"h: " << t->have_piece(r.piece) << " | "
 				"block_limit: " << t->block_size() << " ]\n";
@@ -1345,15 +1345,10 @@ namespace libtorrent
 				// remove the request that just finished
 				// from the download queue plus the
 				// skipped blocks.
-				m_download_queue.erase(m_download_queue.begin()
-					, boost::next(b));
+				m_download_queue.erase(m_download_queue.begin(), b);
+				b = m_download_queue.begin();
+				TORRENT_ASSERT(*b == block_finished);
 			}
-			else
-			{
-				m_download_queue.erase(b);
-			}
-
-			t->cancel_block(block_finished);
 		}
 		else
 		{
@@ -1380,6 +1375,7 @@ namespace libtorrent
 		{
 			t->received_redundant_data(p.length);
 
+			m_download_queue.erase(b);
 			request_a_block(*t, *this);
 			send_block_requests();
 			return;
@@ -1389,10 +1385,21 @@ namespace libtorrent
 			, self(), _1, _2, p, t));
 		m_outstanding_writing_bytes += p.length;
 		TORRENT_ASSERT(!m_reading);
+		m_download_queue.erase(b);
+
+		// did we request this block from any other peers?
+		bool multi = picker.num_peers(block_finished) > 1;
+
 		picker.mark_as_writing(block_finished, peer_info_struct());
+
+		// if we requested this block from other peers, cancel it now
+		if (multi) t->cancel_block(block_finished);
+
 #ifndef NDEBUG
 		t->check_invariant();
 #endif
+		request_a_block(*t, *this);
+		send_block_requests();
 	}
 
 	void peer_connection::on_disk_write_complete(int ret, disk_io_job const& j
@@ -1707,6 +1714,9 @@ namespace libtorrent
 		TORRENT_ASSERT(block.block_index < t->torrent_file().piece_size(block.piece_index));
 		TORRENT_ASSERT(!t->picker().is_requested(block) || (t->picker().num_peers(block) > 0));
 		TORRENT_ASSERT(!t->have_piece(block.piece_index));
+		TORRENT_ASSERT(std::find(m_download_queue.begin(), m_download_queue.end(), block) == m_download_queue.end());
+		TORRENT_ASSERT(std::find(m_request_queue.begin(), m_request_queue.end(), block) == m_request_queue.end());
+
 
 		piece_picker::piece_state_t state;
 		peer_speed_t speed = peer_speed();
@@ -1777,7 +1787,7 @@ namespace libtorrent
 
 		int block_offset = block.block_index * t->block_size();
 		int block_size
-			= (std::min)((int)t->torrent_file().piece_size(block.piece_index)-block_offset,
+			= (std::min)(t->torrent_file().piece_size(block.piece_index)-block_offset,
 			t->block_size());
 		TORRENT_ASSERT(block_size > 0);
 		TORRENT_ASSERT(block_size <= t->block_size());
@@ -1889,7 +1899,7 @@ namespace libtorrent
 			piece_block block = m_request_queue.front();
 
 			int block_offset = block.block_index * t->block_size();
-			int block_size = (std::min)((int)t->torrent_file().piece_size(
+			int block_size = (std::min)(t->torrent_file().piece_size(
 				block.piece_index) - block_offset, t->block_size());
 			TORRENT_ASSERT(block_size > 0);
 			TORRENT_ASSERT(block_size <= t->block_size());
@@ -1935,7 +1945,7 @@ namespace libtorrent
 #endif
 
 					block_offset = block.block_index * t->block_size();
-					block_size = (std::min)((int)t->torrent_file().piece_size(
+					block_size = (std::min)(t->torrent_file().piece_size(
 						block.piece_index) - block_offset, t->block_size());
 					TORRENT_ASSERT(block_size > 0);
 					TORRENT_ASSERT(block_size <= t->block_size());
@@ -1980,7 +1990,7 @@ namespace libtorrent
 
 	void close_socket_ignore_error(boost::shared_ptr<socket_type> s)
 	{
-		try { s->close(); } catch (std::exception& e) {}
+		try { s->close(); } catch (std::exception&) {}
 	}
 
 	void peer_connection::timed_out()
@@ -2392,6 +2402,7 @@ namespace libtorrent
 
 		if (ret != r.length || m_torrent.expired())
 		{
+			if (j.buffer) m_ses.free_disk_buffer(j.buffer);
 			boost::shared_ptr<torrent> t = m_torrent.lock();
 			if (!t)
 			{
@@ -2484,7 +2495,7 @@ namespace libtorrent
 			if (m_bandwidth_limit[upload_channel].max_assignable() > 0)
 			{
 #ifdef TORRENT_VERBOSE_LOGGING
-				(*m_logger) << "req bandwidth [ " << upload_channel << " ]\n";
+				(*m_logger) << time_now_string() << " *** REQUEST_BANDWIDTH [ upload ]\n";
 #endif
 
 				TORRENT_ASSERT(!m_writing);
@@ -2496,7 +2507,18 @@ namespace libtorrent
 			return;
 		}
 
-		if (!can_write()) return;
+		if (!can_write())
+		{
+#ifdef TORRENT_VERBOSE_LOGGING
+			(*m_logger) << time_now_string() << " *** CANNOT WRITE ["
+				" quota: " << m_bandwidth_limit[download_channel].quota_left() <<
+				" ignore: " << (m_ignore_bandwidth_limits?"yes":"no") <<
+				" buf: " << m_send_buffer.size() <<
+				" connecting: " << (m_connecting?"yes":"no") <<
+				" ]\n";
+#endif
+			return;
+		}
 
 		TORRENT_ASSERT(!m_writing);
 
@@ -2511,7 +2533,7 @@ namespace libtorrent
 			TORRENT_ASSERT(amount_to_send > 0);
 
 #ifdef TORRENT_VERBOSE_LOGGING
-			(*m_logger) << "async_write " << amount_to_send << " bytes\n";
+			(*m_logger) << time_now_string() << " *** ASYNC_WRITE [ bytes: " << amount_to_send << " ]\n";
 #endif
 			std::list<asio::const_buffer> const& vec = m_send_buffer.build_iovec(amount_to_send);
 			m_socket->async_write_some(vec, bind(&peer_connection::on_send_data, self(), _1, _2));
@@ -2527,7 +2549,7 @@ namespace libtorrent
 		INVARIANT_CHECK;
 
 #ifdef TORRENT_VERBOSE_LOGGING
-		(*m_logger) << "setup_receive: reading = " << m_reading << "\n";
+		(*m_logger) << time_now_string() << " *** SETUP_RECEIVE [ reading: " << (m_reading?"yes":"no") << "]\n";
 #endif
 		if (m_reading) return;
 
@@ -2541,7 +2563,7 @@ namespace libtorrent
 			if (m_bandwidth_limit[download_channel].max_assignable() > 0)
 			{
 #ifdef TORRENT_VERBOSE_LOGGING
-				(*m_logger) << "req bandwidth [ " << download_channel << " ]\n";
+				(*m_logger) << time_now_string() << " *** REQUEST_BANDWIDTH [ download ]\n";
 #endif
 				m_reading = true;
 				t->request_bandwidth(download_channel, self(), m_priority);
@@ -2549,7 +2571,18 @@ namespace libtorrent
 			return;
 		}
 		
-		if (!can_read()) return;
+		if (!can_read())
+		{
+#ifdef TORRENT_VERBOSE_LOGGING
+			(*m_logger) << time_now_string() << " *** CANNOT READ ["
+				" quota: " << m_bandwidth_limit[download_channel].quota_left() <<
+				" ignore: " << (m_ignore_bandwidth_limits?"yes":"no") <<
+				" outstanding: " << m_outstanding_writing_bytes <<
+				" outstanding-limit: " << m_ses.settings().max_outstanding_disk_bytes_per_connection <<
+				" ]\n";
+#endif
+			return;
+		}
 
 		TORRENT_ASSERT(m_packet_size > 0);
 		int max_receive = m_packet_size - m_recv_pos;
@@ -2564,7 +2597,7 @@ namespace libtorrent
 
 		TORRENT_ASSERT(can_read());
 #ifdef TORRENT_VERBOSE_LOGGING
-		(*m_logger) << "async_read " << max_receive << " bytes\n";
+		(*m_logger) << time_now_string() << " *** ASYNC_READ [ max: " << max_receive << " bytes ]\n";
 #endif
 		m_socket->async_read_some(asio::buffer(&m_recv_buffer[m_recv_pos]
 			, max_receive), bind(&peer_connection::on_receive_data, self(), _1, _2));
@@ -2781,7 +2814,7 @@ namespace libtorrent
 				m_ses.settings().max_outstanding_disk_bytes_per_connection;
 		
 #if defined(TORRENT_VERBOSE_LOGGING)
-		(*m_logger) << "*** can_read() " << ret << " reading: " << m_reading << "\n";
+		(*m_logger) << time_now_string() << " *** can_read() " << (ret?"yes":"no") << " reading: " << m_reading << "\n";
 #endif
 		
 		return ret;
@@ -2798,7 +2831,13 @@ namespace libtorrent
 
 		m_connection_ticket = ticket;
 		boost::shared_ptr<torrent> t = m_torrent.lock();
-		TORRENT_ASSERT(t);
+		if (!t || m_disconnecting)
+		{
+			m_ses.m_half_open.done(m_connection_ticket);
+			m_connecting = false;
+			disconnect();
+			return;
+		}
 
 		m_queued = false;
 		TORRENT_ASSERT(m_connecting);
