@@ -294,6 +294,7 @@ namespace libtorrent
 					|| std::find(rq.begin(), rq.end(), *i) != rq.end())
 					continue;
 	
+				TORRENT_ASSERT(p.num_peers(*i) > 0);
 				busy_pieces.push_back(*i);
 				continue;
 			}
@@ -333,6 +334,8 @@ namespace libtorrent
 		p.piece_info(i->piece_index, st);
 		TORRENT_ASSERT(st.requested + st.finished + st.writing == p.blocks_in_piece(i->piece_index));
 #endif
+		TORRENT_ASSERT(p.is_requested(*i));
+		TORRENT_ASSERT(p.num_peers(*i) > 0);
 		c.add_request(*i);
 		c.send_block_requests();
 	}
@@ -499,10 +502,7 @@ namespace libtorrent
 				= transferred_amount / (connected_time_in_seconds + 1);
 
 			// prefer to disconnect uninteresting peers, and secondly slow peers
-			if (transfer_rate <= slowest_transfer_rate
-				|| (disconnect_peer != m_peers.end()
-				&& disconnect_peer->second.connection->is_interesting()
-				&& !c->is_interesting()))
+			if (transfer_rate <= slowest_transfer_rate)
 			{
 				slowest_transfer_rate = transfer_rate;
 				disconnect_peer = i;
@@ -522,7 +522,18 @@ namespace libtorrent
 
 		int max_failcount = m_torrent->settings().max_failcount;
 		int min_reconnect_time = m_torrent->settings().min_reconnect_time;
+		int min_cidr_distance = (std::numeric_limits<int>::max)();
 		bool finished = m_torrent->is_finished();
+		address external_ip = m_torrent->session().m_external_address;
+
+		if (external_ip == address())
+		{
+			// set external_ip to a random value, to
+			// radomize which peers we prefer
+			address_v4::bytes_type bytes;
+			std::generate(bytes.begin(), bytes.end(), &std::rand);
+			external_ip = address_v4(bytes);
+		}
 
 		aux::session_impl& ses = m_torrent->session();
 
@@ -546,15 +557,29 @@ namespace libtorrent
 
 			TORRENT_ASSERT(i->second.connected <= now);
 
-			if (i->second.connected <= min_connect_time)
-			{
-				min_connect_time = i->second.connected;
-				candidate = i;
-			}
+			if (i->second.connected > min_connect_time) continue;
+			int distance = cidr_distance(external_ip, i->second.ip.address());
+			if (distance > min_cidr_distance) continue;
+
+			min_cidr_distance = distance;
+			min_connect_time = i->second.connected;
+			candidate = i;
 		}
 		
 		TORRENT_ASSERT(min_connect_time <= now);
 
+#if defined TORRENT_LOGGING || defined TORRENT_VERBOSE_LOGGING
+		if (candidate != m_peers.end())
+		{
+			(*m_torrent->session().m_logger) << "*** FOUND CONNECTION CANDIDATE ["
+				" ip: " << candidate->second.ip <<
+				" d: " << min_cidr_distance <<
+				" external: " << external_ip <<
+				" t: " << total_seconds(time_now() - min_connect_time) <<
+				" ]\n";
+		}
+#endif
+
 		return candidate;
 	}
 /*
@@ -1086,15 +1111,6 @@ namespace libtorrent
 				if (flags & 0x01) i->second.pe_support = true;
 #endif
 				if (flags & 0x02) i->second.seed = true;
-
-				// try to send a DHT ping to this peer
-				// as well, to figure out if it supports
-				// DHT (uTorrent and BitComet doesn't
-				// advertise support)
-#ifndef TORRENT_DISABLE_DHT
-				udp::endpoint node(remote.address(), remote.port());
-				m_torrent->session().add_dht_node(node);
-#endif
 			}
 			else
 			{
