@@ -541,9 +541,9 @@ namespace libtorrent
 		for (entry::list_type::iterator i = l.begin();
 			i != l.end(); ++i)
 		{
-			file_sizes.push_back(std::pair<size_type, std::time_t>(
+			file_sizes.push_back(std::make_pair(
 				i->list().front().integer()
-				, i->list().back().integer()));
+				, std::time_t(i->list().back().integer())));
 		}
 
 		if (file_sizes.empty())
@@ -581,7 +581,7 @@ namespace libtorrent
 			// the resume data says we have the entire torrent
 			// make sure the file sizes are the right ones
 			for (torrent_info::file_iterator i = m_info->begin_files(true)
-					, end(m_info->end_files(true)); i != end; ++i, ++fs)
+				, end(m_info->end_files(true)); i != end; ++i, ++fs)
 			{
 				if (i->size != fs->first)
 				{
@@ -591,7 +591,6 @@ namespace libtorrent
 					return false;
 				}
 			}
-			return true;
 		}
 
 		return match_filesizes(*m_info, m_save_path, file_sizes
@@ -819,7 +818,7 @@ namespace libtorrent
 					== file_iter->path);
 #endif
 
-				size_type actual_read = in->read(buf + buf_pos, read_bytes);
+				int actual_read = int(in->read(buf + buf_pos, read_bytes));
 
 				if (read_bytes != actual_read)
 				{
@@ -998,6 +997,9 @@ namespace libtorrent
 		, m_io_thread(io)
 		, m_torrent(torrent)
 	{
+#ifndef NDEBUG
+		m_resume_data_verified = false;
+#endif
 	}
 
 	piece_manager::~piece_manager()
@@ -1011,6 +1013,9 @@ namespace libtorrent
 
 	bool piece_manager::verify_resume_data(entry& rd, std::string& error)
 	{
+#ifndef NDEBUG
+		m_resume_data_verified = true;
+#endif
 		return m_storage->verify_resume_data(rd, error);
 	}
 
@@ -1144,14 +1149,14 @@ namespace libtorrent
 			for (last = m_slot_to_piece.rbegin();
 				last != m_slot_to_piece.rend(); ++last)
 			{
-				if (*last != unallocated && have[*last]) break;
+				if (*last != unallocated) break;
 			}
 
 			for (std::vector<int>::const_iterator i =
 				m_slot_to_piece.begin();
 				i != last.base(); ++i)
 			{
-				p.push_back(have[*i] ? *i : unassigned);
+				p.push_back((*i >= 0) ? *i : unassigned);
 			}
 		}
 		else
@@ -1473,6 +1478,7 @@ namespace libtorrent
 		if (!data.piece_map.empty()
 			&& int(data.piece_map.size()) <= m_info->num_pieces())
 		{
+			TORRENT_ASSERT(m_resume_data_verified);
 			for (int i = 0; i < (int)data.piece_map.size(); ++i)
 			{
 				m_slot_to_piece[i] = data.piece_map[i];
@@ -1696,8 +1702,8 @@ namespace libtorrent
 
 			m_piece_data.resize(int(m_info->piece_length()));
 			int piece_size = int(m_info->piece_size(m_current_slot));
-			int num_read = m_storage->read(&m_piece_data[0]
-				, m_current_slot, 0, piece_size);
+			int num_read = int(m_storage->read(&m_piece_data[0]
+				, m_current_slot, 0, piece_size));
 
 			// if the file is incomplete, skip the rest of it
 			if (num_read != piece_size)
