@@ -102,7 +102,7 @@ namespace
 		// if pieces are too small, adjust the block size
 		if (i.piece_length() < default_block_size)
 		{
-			return static_cast<int>(i.piece_length());
+			return i.piece_length();
 		}
 
 		// otherwise, go with the default
@@ -337,7 +337,7 @@ namespace libtorrent
 	peer_request torrent::to_req(piece_block const& p)
 	{
 		int block_offset = p.block_index * m_block_size;
-		int block_size = (std::min)((int)torrent_file().piece_size(
+		int block_size = (std::min)(torrent_file().piece_size(
 			p.piece_index) - block_offset, m_block_size);
 		TORRENT_ASSERT(block_size > 0);
 		TORRENT_ASSERT(block_size <= m_block_size);
@@ -403,8 +403,8 @@ namespace libtorrent
 		m_storage = m_owning_storage.get();
 		m_block_size = calculate_block_size(*m_torrent_file, m_default_block_size);
 		m_picker.reset(new piece_picker(
-			static_cast<int>(m_torrent_file->piece_length() / m_block_size)
-			, static_cast<int>((m_torrent_file->total_size()+m_block_size-1)/m_block_size)));
+			m_torrent_file->piece_length() / m_block_size
+			, int((m_torrent_file->total_size()+m_block_size-1)/m_block_size)));
 
 		std::vector<std::string> const& url_seeds = m_torrent_file->url_seeds();
 		std::copy(url_seeds.begin(), url_seeds.end(), std::inserter(m_web_seeds
@@ -775,8 +775,7 @@ namespace libtorrent
 		const std::vector<piece_picker::downloading_piece>& dl_queue
 			= m_picker->get_download_queue();
 
-		const int blocks_per_piece = static_cast<int>(
-			piece_size / m_block_size);
+		const int blocks_per_piece = piece_size / m_block_size;
 
 		for (std::vector<piece_picker::downloading_piece>::const_iterator i =
 			dl_queue.begin(); i != dl_queue.end(); ++i)
@@ -1245,8 +1244,9 @@ namespace libtorrent
 		TORRENT_ASSERT(index >= 0);
 		TORRENT_ASSERT(index < m_torrent_file->num_pieces());
 
+		bool was_finished = is_finished();
 		bool filter_updated = m_picker->set_piece_priority(index, priority);
-		if (filter_updated) update_peer_interest();
+		if (filter_updated) update_peer_interest(was_finished);
 	}
 
 	int torrent::piece_priority(int index) const
@@ -1276,6 +1276,7 @@ namespace libtorrent
 
 		int index = 0;
 		bool filter_updated = false;
+		bool was_finished = is_finished();
 		for (std::vector<int>::const_iterator i = pieces.begin()
 			, end(pieces.end()); i != end; ++i, ++index)
 		{
@@ -1283,7 +1284,7 @@ namespace libtorrent
 			TORRENT_ASSERT(*i <= 7);
 			filter_updated |= m_picker->set_piece_priority(index, *i);
 		}
-		if (filter_updated) update_peer_interest();
+		if (filter_updated) update_peer_interest(was_finished);
 	}
 
 	void torrent::piece_priorities(std::vector<int>& pieces) const
@@ -1326,6 +1327,8 @@ namespace libtorrent
 
 		if (m_torrent_file->num_pieces() == 0) return;
 
+		bool was_finished = is_finished();
+
 		int piece_length = m_torrent_file->piece_length();
 		// initialize the piece priorities to 0, then only allow
 		// setting higher priorities
@@ -1349,14 +1352,19 @@ namespace libtorrent
 				, bind(&set_if_greater, _1, files[i]));
 		}
 		prioritize_pieces(pieces);
-		update_peer_interest();
+		update_peer_interest(was_finished);
 	}
 
+	// this is called when piece priorities have been updated
 	// updates the interested flag in peers
-	void torrent::update_peer_interest()
+	void torrent::update_peer_interest(bool was_finished)
 	{
 		for (peer_iterator i = begin(); i != end(); ++i)
 			(*i)->update_interest();
+		
+		// the torrent just became finished
+		if (is_finished() && !was_finished)
+			finished();
 	}
 
 	void torrent::filter_piece(int index, bool filter)
@@ -1371,8 +1379,9 @@ namespace libtorrent
 		TORRENT_ASSERT(index >= 0);
 		TORRENT_ASSERT(index < m_torrent_file->num_pieces());
 
+		bool was_finished = is_finished();
 		m_picker->set_piece_priority(index, filter ? 1 : 0);
-		update_peer_interest();
+		update_peer_interest(was_finished);
 	}
 
 	void torrent::filter_pieces(std::vector<bool> const& bitmask)
@@ -1385,6 +1394,7 @@ namespace libtorrent
 
 		TORRENT_ASSERT(m_picker.get());
 
+		bool was_finished = is_finished();
 		int index = 0;
 		for (std::vector<bool>::const_iterator i = bitmask.begin()
 			, end(bitmask.end()); i != end; ++i, ++index)
@@ -1395,7 +1405,7 @@ namespace libtorrent
 			else
 				m_picker->set_piece_priority(index, 1);
 		}
-		update_peer_interest();
+		update_peer_interest(was_finished);
 	}
 
 	bool torrent::is_piece_filtered(int index) const
@@ -1534,6 +1544,8 @@ namespace libtorrent
 
 	void torrent::cancel_block(piece_block block)
 	{
+		INVARIANT_CHECK;
+
 		for (peer_iterator i = m_connections.begin()
 			, end(m_connections.end()); i != end; ++i)
 		{
@@ -1618,6 +1630,54 @@ namespace libtorrent
 		(*m_ses.m_logger) << time_now_string() << " resolving web seed: " << url << "\n";
 #endif
 
+		std::string protocol;
+		std::string auth;
+		std::string hostname;
+		int port;
+		std::string path;
+		boost::tie(protocol, auth, hostname, port, path)
+			= parse_url_components(url);
+
+#ifdef TORRENT_USE_OPENSSL
+		if (protocol != "http" && protocol != "https")
+#else
+		if (protocol != "http")
+#endif
+		{
+			if (m_ses.m_alerts.should_post(alert::warning))
+			{
+				m_ses.m_alerts.post_alert(
+					url_seed_alert(get_handle(), url, "unknown protocol"));
+			}
+			// never try it again
+			remove_url_seed(url);
+			return;
+		}
+
+		if (hostname.empty())
+		{
+			if (m_ses.m_alerts.should_post(alert::warning))
+			{
+				m_ses.m_alerts.post_alert(
+					url_seed_alert(get_handle(), url, "invalid hostname"));
+			}
+			// never try it again
+			remove_url_seed(url);
+			return;
+		}
+
+		if (port == 0)
+		{
+			if (m_ses.m_alerts.should_post(alert::warning))
+			{
+				m_ses.m_alerts.post_alert(
+					url_seed_alert(get_handle(), url, "invalid port"));
+			}
+			// never try it again
+			remove_url_seed(url);
+			return;
+		}
+
 		m_resolving_web_seeds.insert(url);
 		proxy_settings const& ps = m_ses.web_seed_proxy();
 		if (ps.type == proxy_settings::http
@@ -1631,16 +1691,6 @@ namespace libtorrent
 		}
 		else
 		{
-			std::string protocol;
-			std::string auth;
-			std::string hostname;
-			int port;
-			std::string path;
-			boost::tie(protocol, auth, hostname, port, path)
-				= parse_url_components(url);
-
-			// TODO: should auth be used here?
-
 			tcp::resolver::query q(hostname, boost::lexical_cast<std::string>(port));
 			m_host_resolver.async_resolve(q, m_ses.m_strand.wrap(
 				bind(&torrent::on_name_lookup, shared_from_this(), _1, _2, url
@@ -1700,7 +1750,7 @@ namespace libtorrent
 		m_host_resolver.async_resolve(q, m_ses.m_strand.wrap(
 			bind(&torrent::on_name_lookup, shared_from_this(), _1, _2, url, a)));
 	}
-	catch (std::exception& exc)
+	catch (std::exception&)
 	{
 		TORRENT_ASSERT(false);
 	};
@@ -1985,7 +2035,7 @@ namespace libtorrent
 			pi.finished = (int)i->finished;
 			pi.writing = (int)i->writing;
 			pi.requested = (int)i->requested;
-			int piece_size = torrent_file().piece_size(i->index);
+			int piece_size = int(torrent_file().piece_size(i->index));
 			for (int j = 0; j < pi.blocks_in_piece; ++j)
 			{
 				block_info& bi = pi.blocks[j];
@@ -2176,7 +2226,7 @@ namespace libtorrent
 #endif
 			m_policy.new_connection(*p);
 		}
-		catch (std::exception& e)
+		catch (std::exception&)
 		{
 			throw;
 		}
@@ -2862,6 +2912,13 @@ namespace libtorrent
 			m_storage->async_release_files(
 				bind(&torrent::on_torrent_paused, shared_from_this(), _1, _2));
 		}
+		else
+		{
+			if (alerts().should_post(alert::warning))
+			{
+				alerts().post_alert(torrent_paused_alert(get_handle(), "torrent paused"));
+			}
+		}
 	}
 
 	void torrent::resume()
@@ -2965,6 +3022,7 @@ namespace libtorrent
 			}
 			catch (std::exception& e)
 			{
+				(void)e;
 #ifdef TORRENT_VERBOSE_LOGGING
 				(*p->m_logger) << "**ERROR**: " << e.what() << "\n";
 #endif
@@ -3051,8 +3109,8 @@ namespace libtorrent
 			size_type done = 0;
 			while (size > 0)
 			{
-				size_type bytes_step = (std::min)(m_torrent_file->piece_size(ret.piece)
-					- ret.start, size);
+				size_type bytes_step = (std::min)(size_type(m_torrent_file->piece_size(ret.piece)
+					- ret.start), size);
 				if (m_have_pieces[ret.piece]) done += bytes_step;
 				++ret.piece;
 				ret.start = 0;
@@ -3175,7 +3233,7 @@ namespace libtorrent
 
 		if (st.total_wanted == 0) st.progress = 1.f;
 		else st.progress = st.total_wanted_done
-			/ static_cast<double>(st.total_wanted);
+			/ static_cast<float>(st.total_wanted);
 
 		st.pieces = &m_have_pieces;
 		st.num_pieces = m_num_pieces;
