@@ -261,6 +261,12 @@ namespace libtorrent
 		TORRENT_FORWARD_RETURN(is_seed(), false);
 	}
 
+	bool torrent_handle::is_finished() const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD_RETURN(is_finished(), false);
+	}
+
 	bool torrent_handle::is_paused() const
 	{
 		INVARIANT_CHECK;
@@ -283,7 +289,7 @@ namespace libtorrent
 		, std::string const& password) const
 	{
 		INVARIANT_CHECK;
-		TORRENT_FORWARD(resume());
+		TORRENT_FORWARD(set_tracker_login(name, password));
 	}
 
 	void torrent_handle::file_progress(std::vector<float>& progress)
@@ -485,7 +491,7 @@ namespace libtorrent
 		session_impl::mutex_t::scoped_lock l1(m_ses->m_mutex);
 		mutex::scoped_lock l2(m_chk->m_mutex);
 		torrent* t = find_torrent(m_ses, m_chk, m_info_hash);
-		return t;
+		return t != 0;
 	}
 
 	entry torrent_handle::write_resume_data() const
@@ -569,7 +575,7 @@ namespace libtorrent
 					TORRENT_ASSERT(bits == 8 || j == num_bitmask_bytes - 1);
 				}
 				piece_struct["bitmask"] = bitmask;
-
+/*
 				TORRENT_ASSERT(t->filesystem().slot_for(i->index) >= 0);
 				unsigned long adler
 					= t->filesystem().piece_crc(
@@ -578,7 +584,7 @@ namespace libtorrent
 						, i->info);
 
 				piece_struct["adler32"] = adler;
-
+*/
 				// push the struct onto the unfinished-piece list
 				up.push_back(piece_struct);
 			}
@@ -596,6 +602,8 @@ namespace libtorrent
 		
 		policy& pol = t->get_policy();
 
+		int max_failcount = t->settings().max_failcount;
+
 		for (policy::iterator i = pol.begin_peer()
 			, end(pol.end_peer()); i != end; ++i)
 		{
@@ -619,6 +627,9 @@ namespace libtorrent
 			// been banned, don't save it.
 			if (i->second.type == policy::peer::not_connectable) continue;
 
+			// don't save peers that doesn't work
+			if (i->second.failcount >= max_failcount) continue;
+
 			tcp::endpoint ip = i->second.ip;
 			entry peer(entry::dictionary_t);
 			peer["ip"] = ip.address().to_string(ec);
