@@ -76,27 +76,13 @@ upnp::upnp(io_service& ios, connection_queue& cc
 	, m_refresh_timer(ios)
 	, m_disabled(false)
 	, m_closing(false)
+	, m_ignore_outside_network(ignore_nonrouters)
 	, m_cc(cc)
 {
 #ifdef TORRENT_UPNP_LOGGING
 	m_log.open("upnp.log", std::ios::in | std::ios::out | std::ios::trunc);
 #endif
 	m_retry_count = 0;
-
-	if (ignore_nonrouters)
-	{
-		asio::error_code ec;
-		std::vector<address> const& net = enum_net_interfaces(m_io_service, ec);
-		m_filter.reserve(net.size());
-		for (std::vector<address>::const_iterator i = net.begin()
-			, end(net.end()); i != end; ++i)
-		{
-			asio::error_code e;
-			address a = router_for_interface(*i, e);
-			if (e || is_loopback(a)) continue;
-			m_filter.push_back(a);
-		}
-	}
 }
 
 upnp::~upnp()
@@ -230,6 +216,7 @@ try
 			}
 			catch (std::exception& e)
 			{
+				(void)e;
 #ifdef TORRENT_UPNP_LOGGING
 				m_log << time_now_string()
 					<< " *** Connection failed to: " << d.url
@@ -281,14 +268,29 @@ try
 	Server:Microsoft-Windows-NT/5.1 UPnP/1.0 UPnP-Device-Host/1.0
 
 */
-	if (!m_filter.empty() && std::find(m_filter.begin(), m_filter.end()
-		, from.address()) == m_filter.end())
+	asio::error_code ec;
+	if (m_ignore_outside_network && !in_local_network(m_io_service, from.address(), ec))
 	{
 		// this upnp device is filtered because it's not in the
 		// list of configured routers
 #ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string() << " <== (" << from << ") Rootdevice "
-			"ignored because it's not out router" << std::endl;
+		if (ec)
+		{
+			m_log << time_now_string() << " <== (" << from << ") error: "
+				<< ec.message() << std::endl;
+		}
+		else
+		{
+			std::vector<ip_interface> const& net = enum_net_interfaces(m_io_service, ec);
+			m_log << time_now_string() << " <== (" << from << ") UPnP device "
+				"ignored because it's not on our network ";
+			for (std::vector<ip_interface>::const_iterator i = net.begin()
+				, end(net.end()); i != end; ++i)
+			{
+				m_log << "(" << i->interface_address << ", " << i->netmask << ") ";
+			}
+			m_log << std::endl;
+		}
 #endif
 		return;
 	}
@@ -301,6 +303,7 @@ try
 	}
 	catch (std::exception& e)
 	{
+		(void)e;
 #ifdef TORRENT_UPNP_LOGGING
 		m_log << time_now_string()
 			<< " <== (" << from << ") Rootdevice responded with incorrect HTTP packet. Ignoring device (" << e.what() << ")" << std::endl;
@@ -448,6 +451,7 @@ try
 				}
 				catch (std::exception& e)
 				{
+					(void)e;
 #ifdef TORRENT_UPNP_LOGGING
 					m_log << time_now_string()
 						<< " *** Connection failed to: " << d.url
@@ -470,6 +474,8 @@ void upnp::post(upnp::rootdevice const& d, std::string const& soap
 	, std::string const& soap_action)
 {
 	TORRENT_ASSERT(d.magic == 1337);
+	TORRENT_ASSERT(d.upnp_connection);
+
 	std::stringstream header;
 	
 	header << "POST " << d.control_url << " HTTP/1.1\r\n"
@@ -490,6 +496,17 @@ void upnp::post(upnp::rootdevice const& d, std::string const& soap
 void upnp::create_port_mapping(http_connection& c, rootdevice& d, int i)
 {
 	TORRENT_ASSERT(d.magic == 1337);
+
+	if (!d.upnp_connection)
+	{
+		TORRENT_ASSERT(d.disabled);
+#ifdef TORRENT_UPNP_LOGGING
+		m_log << time_now_string() << " *** mapping (" << i
+			<< ") aborted" << std::endl;
+#endif
+		return;
+	}
+	
 	std::string soap_action = "AddPortMapping";
 
 	std::stringstream soap;
@@ -547,6 +564,17 @@ void upnp::map_port(rootdevice& d, int i)
 void upnp::delete_port_mapping(rootdevice& d, int i)
 {
 	TORRENT_ASSERT(d.magic == 1337);
+
+	if (!d.upnp_connection)
+	{
+		TORRENT_ASSERT(d.disabled);
+#ifdef TORRENT_UPNP_LOGGING
+		m_log << time_now_string() << " *** unmapping (" << i
+			<< ") aborted" << std::endl;
+#endif
+		return;
+	}
+
 	std::stringstream soap;
 	
 	std::string soap_action = "DeletePortMapping";
