@@ -75,6 +75,7 @@ PREF_FUNCTIONS = {
     "max_active_torrents" : None, # no need for a function, applied constantly
     "max_upload_slots_global" : deluge_core.set_max_upload_slots_global,
     "auto_seed_ratio" : None, # no need for a function, applied constantly
+    "auto_seed_time" : None, # no need for a function, applied constantly
     "max_download_speed_bps" : deluge_core.set_download_rate_limit,
     "max_upload_speed_bps" : deluge_core.set_upload_rate_limit,
     "enable_dht" : None, # not a normal pref in that is is applied only on start
@@ -164,16 +165,17 @@ class torrent_info:
         self.compact = compact
         self.user_paused = user_paused
         self.uploaded_memory = 0
-        self.initial_uploaded_memory = 0
         self.upload_rate_limit = 0
         self.download_rate_limit = 0
         self.webseed_urls = []
         self.desired_ratio = 1.0
         self.trackers = ""
         self.trackers_changed = 0
+        self.seed_time = 0.0
 
         self.delete_me = False # set this to true, to delete it on next sync
         self.del_data = False # set this to true, to delete data on next sync
+        self.del_torrent = False # set this to true, to delete torrent on next sync
 
 # The persistent state of the torrent system. Everything in this will be pickled
 
@@ -269,10 +271,17 @@ class Manager:
                 
                 if isinstance(state.torrents, list):
                     # One time convert of old torrents list to dict
-                    state.torrents = dict((x, None) for x in 
-                                                   state.torrents)
-                # Add torrents to core and unique_IDs
+                    state.torrents = dict((x, None) for x in state.torrents)
+                
+                fr_sorted = []
                 for torrent in state.torrents:
+                    if os.path.exists(torrent.filename + ".fastresume"):
+                        fr_sorted.insert(0, torrent)
+                    else:
+                        fr_sorted.append(torrent)
+                        
+                # Add torrents to core and unique_IDs
+                for torrent in fr_sorted:
                     if not os.path.exists(torrent.filename):
                         print "Missing file: %s" % torrent.filename
                         continue
@@ -295,7 +304,7 @@ class Manager:
                         except:
                             pass
                         else:
-                            if not torrent_state['is_seed']:
+                            if not torrent_state['is_finished']:
                                 try:
                                     os.remove(self.unique_IDs[unique_ID].filename + ".fastresume")
                                 except:
@@ -335,46 +344,40 @@ class Manager:
         print "Saving fastresume data..."
         self.save_fastresume_data()
 
-        # save upload memory
-        print "Saving uploaded memory..."
-        self.save_upmem()
-
         # Shutdown torrent core
         print "Quitting the core..."
         deluge_core.quit()
 
         #kill dbus on windows
-        import platform
-        if platform.system() == "Windows":
-            import os
-            os.popen4('tskill.exe dbus-daemon-deluge')
-        elif platform.system() == "Microsoft":
-            import os
-            os.popen4('taskkill.exe /IM dbus-daemon-deluge.exe /F')
+        if common.windows_check():
+            if platform.win32_ver()[1] > '6':
+                os.popen4('taskkill.exe /IM dbus-daemon-deluge.exe /F')
+            else:
+                os.popen4('tskill.exe dbus-daemon-deluge')
 
     def pickle_state(self):
+        print "save uploaded memory"
+        state = persistent_state()
+        state.torrents = dict(self.state.torrents.iteritems())
+        state.queue = self.state.queue[:]
+        
+        for torrent in state.torrents:
+            uid = state.torrents[torrent]
+            try:
+                torrent.uploaded_memory = self.unique_IDs[uid].uploaded_memory +\
+                    self.get_core_torrent_state(uid, False)['total_upload']
+            except AttributeError:
+                torrent.uploaded_memory = 0
         # Pickle the state so if we experience a crash, the latest state is 
         # available
         print "Pickling state..."
         if not os.path.exists(self.base_dir):
             os.makedirs(self.base_dir)
         output = open(os.path.join(self.base_dir, STATE_FILENAME), 'wb')
-        pickle.dump(self.state, output)
+        pickle.dump(state, output)
         output.close()
 
-    def save_upmem(self):
-        # Save the uploaded data from this session to the existing upload memory
-        for unique_ID in self.unique_IDs.keys():
-            # self.get_core_torrent_state purposefully not cached.
-            try:
-                self.unique_IDs[unique_ID].uploaded_memory = \
-                self.unique_IDs[unique_ID].initial_uploaded_memory + \
-                self.get_core_torrent_state(unique_ID, False)['total_upload']
-            except AttributeError:
-                self.unique_IDs[unique_ID].initial_uploaded_memory = 0
-
     # Preference management functions
-
     def get_config(self):
         # This returns the preference object
         return self.config
@@ -460,22 +463,35 @@ class Manager:
         if uid == None:
             for unique_ID in self.unique_IDs:
                 try:
-                    os.remove(self.unique_IDs[unique_ID].filename + ".fastresume")
-                except:
-                    pass
-                try:
-                    deluge_core.save_fastresume(unique_ID, self.unique_IDs[unique_ID].filename)
+                    state = deluge_core.get_torrent_state(unique_ID)
+                    if not state['is_finished'] and state['state'] != 0 and state['state'] != 1:
+                        os.remove(self.unique_IDs[unique_ID].filename + ".fastresume")
+                        deluge_core.save_fastresume(unique_ID, self.unique_IDs[unique_ID].filename)
+                except OSError:
+                    print "no old fastresume to delete"
+                    try:
+                        deluge_core.save_fastresume(unique_ID, self.unique_IDs[unique_ID].filename)
+                    except:
+                        pass
                 except Exception, e:
                     print "Unable to save fastresume: ", e
         else:
+            # Do not save fastresume if torrent is Queued for checking or being checked
             try:
+                state = deluge_core.get_torrent_state(uid)
+                if state['state'] == 0 or state['state'] == 1:
+                    return
                 os.remove(self.unique_IDs[uid].filename + ".fastresume")
-            except:
-                pass
-            try:
                 deluge_core.save_fastresume(uid, self.unique_IDs[uid].filename)
+            except OSError:
+                print "no old fastresume to delete"
+                try:
+                    deluge_core.save_fastresume(uid, self.unique_IDs[uid].filename)
+                except:
+                    pass
             except Exception, e:
                 print "Unable to save fastresume: ", e
+        return True
 
     # State retrieval functions
 
@@ -574,7 +590,7 @@ class Manager:
         for unique_ID in self.unique_IDs:
             torrent_state = self.get_core_torrent_state(unique_ID)
             if torrent_state['progress'] == 1.0:
-                self.remove_torrent_ns(unique_ID, False)
+                self.remove_torrent_ns(unique_ID, False, True)
                 self.removed_unique_ids[unique_ID] = 1
 
         self.sync()
@@ -608,7 +624,7 @@ class Manager:
                torrent_state['is_paused'] and not \
                self.is_user_paused(unique_ID):
                 # This torrent is a seed so skip all the free space checking
-                if torrent_state['is_seed']:
+                if torrent_state['is_finished']:
                     self.resume(unique_ID)
                     continue
                     
@@ -647,7 +663,6 @@ Space:") + " " + nice_free)
         if not self.get_pref('clear_max_ratio_torrents') \
             and self.get_pref('auto_seed_ratio') > 0 \
             and self.get_pref('auto_end_seeding'):
-
             for unique_ID in self.unique_IDs:
                 torrent_state = self.get_core_torrent_state(unique_ID)
                 if (torrent_state["total_wanted"] - torrent_state["total_wanted_done"] == 0) and not torrent_state['is_paused']:
@@ -655,8 +670,20 @@ Space:") + " " + nice_free)
                     if ratio >= self.get_pref('auto_seed_ratio'):
                         self.queue_bottom(unique_ID, enforce_queue=False) # don't recurse!
                         self.set_user_pause(unique_ID, True, enforce_queue=False)
+
+        if not self.get_pref('clear_max_rtime_torrents') \
+            and self.get_pref('auto_seed_time') > 0 \
+            and self.get_pref('auto_time_end_seeding'):
+            for unique_ID in self.unique_IDs:
+                torrent_state = self.get_core_torrent_state(unique_ID)
+                if (torrent_state["total_wanted"] - torrent_state["total_wanted_done"] == 0) and not torrent_state['is_paused']:
+                    seed_time = self.unique_IDs[unique_ID].seed_time 
+                    now = time.time()
+                    if (now - seed_time) >= (self.get_pref('auto_seed_time') * 3600):
+                        self.queue_bottom(unique_ID, enforce_queue=False) # don't recurse!
+                        self.set_user_pause(unique_ID, True, enforce_queue=False)
         
-        if self.get_pref('clear_max_ratio_torrents'):
+        if self.get_pref('clear_max_ratio_torrents') and self.get_pref('auto_end_seeding'):
             for unique_ID in self.unique_IDs.keys():
                 torrent_state = self.get_core_torrent_state(unique_ID)
                 if (torrent_state["total_wanted"] - torrent_state["total_wanted_done"] == 0):
@@ -664,6 +691,15 @@ Space:") + " " + nice_free)
                     if ratio >= self.get_pref('auto_seed_ratio'):
                         self.removed_unique_ids[unique_ID] = 1
                         self.remove_torrent(unique_ID, False, True)
+        if self.get_pref('clear_max_time_torrents') and self.get_pref('auto_time_end_seeding'):
+            for unique_ID in self.unique_IDs.keys():
+                torrent_state = self.get_core_torrent_state(unique_ID)
+                if (torrent_state["total_wanted"] - torrent_state["total_wanted_done"] == 0):
+                    seed_time = self.unique_IDs[unique_ID].seed_time 
+                    now = time.time()
+                    if (now - seed_time) >= (self.get_pref('auto_seed_time') * 3600):
+                        self.removed_unique_ids[unique_ID] = 1
+                        self.remove_torrent(unique_ID, False, True)
     # Event handling
 
     def connect_event(self, event_type, callback):
@@ -704,17 +740,22 @@ Space:") + " " + nice_free)
             if event['event_type'] in self.event_callbacks:
                 for callback in self.event_callbacks[event['event_type']]:
                     callback(event)
+            
+            if event['event_type'] is self.constants['EVENT_TORRENT_PAUSED'] or \
+                event['event_type'] is self.constants['EVENT_FINISHED_CHECKING']:
+                self.save_fastresume_data(event['unique_ID'])
 
-            if event['event_type'] is self.constants['EVENT_FINISHED']: 
+            if event['event_type'] is self.constants['EVENT_FINISHED']:
                 if event['message'] == "torrent has finished downloading":
 
+                    # save fast resume once torrent finishes so as to not recheck
+                    # seed if client crashes
+                    self.save_fastresume_data(event['unique_ID'])
+                    self.unique_IDs[event['unique_ID']].seed_time = time.time()
                     # Queue seeding torrent to bottom if needed
                     if self.get_pref('queue_seeds_to_bottom'):
                         self.queue_bottom(event['unique_ID'])
                     
-                # save fast resume once torrent finishes so as to not recheck
-                # seed if client crashes
-                self.save_fastresume_data(event['unique_ID'])
 
             elif event['event_type'] is self.constants['EVENT_FILE_ERROR']:
                 import gtk
@@ -731,6 +772,9 @@ Space:") + " " + nice_free)
                 self.set_supp_torrent_state_val(event['unique_ID'], 
                                                 "tracker_status",
                                                 _("Announce OK"))
+                if event['num_complete'] == -1 or event['num_incomplete'] == -1:
+                    self.scrape_tracker(event['unique_ID'])
+                    
             elif event['event_type'] is self.constants['EVENT_TRACKER_ALERT']:
                 match = re.search('tracker:\s*".*"\s*(.*)', event["message"])
                 message = match and match.groups()[0] or ""
@@ -960,9 +1004,10 @@ likely the tracker did not responsd in utf-8."
         new_torrent = torrent_info(full_new_name, save_dir, compact, user_paused)
         self.state.torrents[new_torrent] = None
 
-    def remove_torrent_ns(self, unique_ID, data_also):
+    def remove_torrent_ns(self, unique_ID, data_also, torrent_also=False):
         self.unique_IDs[unique_ID].delete_me = True
-        self.unique_IDs[unique_ID].del_data = data_also        
+        self.unique_IDs[unique_ID].del_data = data_also
+        self.unique_IDs[unique_ID].del_torrent = torrent_also
 
     # Sync the state.torrents and unique_IDs lists with the core
     # ___ALL syncing code with the core is here, and ONLY here___
@@ -989,6 +1034,10 @@ likely the tracker did not responsd in utf-8."
                     unique_ID = deluge_core.add_torrent(torrent.filename,
                                                         torrent.save_dir,
                                                         torrent.compact)
+                except DelugeError, e:
+                    del self.state.torrents[torrent]
+                    raise e
+                    continue
                 except Exception, e:
                     print "Unable to add torrent: ", e
                     to_remove.append(torrent)
@@ -1006,7 +1055,7 @@ likely the tracker did not responsd in utf-8."
                 except:
                     pass
                 else:
-                    if not torrent_state['is_seed']:
+                    if not torrent_state['is_finished']:
                         try:
                             os.remove(self.unique_IDs[unique_ID].filename + ".fastresume")
                         except:
@@ -1029,6 +1078,11 @@ likely the tracker did not responsd in utf-8."
         for unique_ID in to_delete:
             del self.state.torrents[self.unique_IDs[unique_ID]]
             self.state.queue.remove(self.unique_IDs[unique_ID])
+            try:
+                if self.unique_IDs[unique_ID].del_torrent:
+                    os.remove(self.unique_IDs[unique_ID].filename)
+            except:
+                pass
             # Remove .fastresume
             try:
                 # Must be after removal of the torrent, because that saves a new .fastresume
@@ -1130,9 +1184,10 @@ likely the tracker did not responsd in utf-8."
 
     def calc_ratio(self, unique_ID, torrent_state):
         try:
-            up = float((self.unique_IDs[unique_ID].initial_uploaded_memory + self.get_core_torrent_state(unique_ID, False)['total_upload']) / 1024)
-        except AttributeError:
             up = float((self.unique_IDs[unique_ID].uploaded_memory + self.get_core_torrent_state(unique_ID, False)['total_upload']) / 1024)
+        except AttributeError:
+            up = 0
+            
         down = float(torrent_state["total_done"] / 1024)
         try:
             ret = up/down
