@@ -42,6 +42,9 @@
 #  undef socklen_t
 # endif
 #endif
+#ifdef DELUGE_CORE_DEBUG
+#include <time.h>
+#endif
 #include <boost/filesystem/exception.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -92,6 +95,8 @@ using namespace libtorrent;
 #define EVENT_BLOCK_FINISHED        17
 #define EVENT_PEER_BLOCKED          18
 #define EVENT_LISTEN_FAILED         19
+#define EVENT_FINISHED_CHECKING     20
+#define EVENT_TORRENT_PAUSED        21
 
 #define STATE_QUEUED                0
 #define STATE_CHECKING              1
@@ -153,12 +158,26 @@ static PyObject *InvalidTorrentError   = NULL;
 
 bool empty_name_check(const std::string & name)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " empty_name_check()" << std::endl;
+#endif
     return 1;
 }
 
 
 long handle_exists(torrent_handle &handle)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " handle_exists()" << std::endl;
+#endif
     for (unsigned long i = 0; i < M_torrents->size(); i++)
         if ((*M_torrents)[i].handle == handle)
             return 1;
@@ -169,6 +188,13 @@ long handle_exists(torrent_handle &handle)
 
 long get_torrent_index(torrent_handle &handle)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " get_torrent_index()" << std::endl;
+#endif
     for (unsigned long i = 0; i < M_torrents->size(); i++)
         if ((*M_torrents)[i].handle == handle)
     {
@@ -181,6 +207,13 @@ long get_torrent_index(torrent_handle &handle)
 
 long get_index_from_unique_ID(long unique_ID)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " get_index_from_unique_ID()" << std::endl;
+#endif
     try{
       for (unsigned long i = 0; i < M_torrents->size(); i++)
           if ((*M_torrents)[i].unique_ID == unique_ID)
@@ -199,6 +232,13 @@ long get_index_from_unique_ID(long unique_ID)
 
 partial_piece_info internal_get_piece_info(torrent_handle h, long piece_index)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " internal_get_piece_info()" << std::endl;
+#endif
     std::vector<partial_piece_info> queue;
     std::vector<partial_piece_info>& q = queue;
     h.get_download_queue(q);
@@ -210,6 +250,13 @@ partial_piece_info internal_get_piece_info(torrent_handle h, long piece_index)
 
 torrent_info internal_get_torrent_info(std::string const& torrent_name)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " internal_get_torrent_info()" << std::endl;
+#endif
     std::ifstream in(torrent_name.c_str(), std::ios_base::binary);
     in.unsetf(std::ios_base::skipws);
     entry e = bdecode(std::istream_iterator<char>(in), std::istream_iterator<char>());
@@ -220,10 +267,16 @@ torrent_info internal_get_torrent_info(std::string const& torrent_name)
 }
 
 long internal_add_torrent(std::string const&    torrent_name,
-float   preferred_ratio,
 bool    compact_mode,
 boost::filesystem::path const& save_path)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " internal_add_torrent()" << std::endl;
+#endif
 
     std::ifstream in(torrent_name.c_str(), std::ios_base::binary);
     in.unsetf(std::ios_base::skipws);
@@ -263,7 +316,6 @@ boost::filesystem::path const& save_path)
         torrent_handle h = M_ses->add_torrent(t, save_path, resume_data, storage_mode);
         //    h.set_max_connections(60); // at some point we should use this
         h.set_max_uploads(-1);
-        h.set_ratio(preferred_ratio);
         h.resolve_countries(true);
         new_torrent.handle = h;
         new_torrent.unique_ID = M_unique_counter;
@@ -287,6 +339,13 @@ boost::filesystem::path const& save_path)
 
 void internal_remove_torrent(long index, int delete_files)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " internal_remove_torrent()" << std::endl;
+#endif
     assert(index < M_torrents->size());
 
     torrent_handle& h = M_torrents->at(index).handle;
@@ -299,6 +358,13 @@ void internal_remove_torrent(long index, int delete_files)
 
 long get_peer_index(tcp::endpoint addr, std::vector<peer_info> const& peers)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " get_peer_index()" << std::endl;
+#endif
     long index = -1;
 
     for (unsigned long i = 0; i < peers.size(); i++)
@@ -310,6 +376,13 @@ long get_peer_index(tcp::endpoint addr, std::vector<peer_info> const& peers)
 
 bool internal_has_piece(std::vector<bool> const& pieces, long index)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " internal_has_piece()" << std::endl;
+#endif
     return pieces[index];
 }
 
@@ -318,6 +391,13 @@ void internal_add_files(torrent_info&   t,
 boost::filesystem::path const& p,
 boost::filesystem::path const& l)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " internal_add_files()" << std::endl;
+#endif
                                  // change default checker, perhaps?
     boost::filesystem::path f(p / l);
     if (is_directory(f))
@@ -331,6 +411,13 @@ boost::filesystem::path const& l)
 
 long count_DHT_peers(entry &state)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " count_DHT_peers()" << std::endl;
+#endif
     long num_peers = 0;
     entry *nodes = state.find_key("nodes");
     if (nodes)
@@ -350,6 +437,13 @@ long count_DHT_peers(entry &state)
 
 static PyObject *torrent_pre_init(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_pre_init()" << std::endl;
+#endif
     if (!PyArg_ParseTuple(args, "OOOOOO", &DelugeError,
         &InvalidEncodingError,
         &SystemError,
@@ -364,6 +458,13 @@ static PyObject *torrent_pre_init(PyObject *self, PyObject *args)
 
 static PyObject *torrent_init(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_init()" << std::endl;
+#endif
     printf("deluge_core; using libtorrent %s. Compiled %s NDEBUG.\r\n",
         LIBTORRENT_VERSION,
 #ifdef NDEBUG
@@ -435,6 +536,8 @@ static PyObject *torrent_init(PyObject *self, PyObject *args)
         "EVENT_BLOCK_FINISHED",               EVENT_BLOCK_FINISHED,
         "EVENT_PEER_BLOCKED",                 EVENT_PEER_BLOCKED,
         "EVENT_LISTEN_FAILED",                EVENT_LISTEN_FAILED,
+        "EVENT_FINISHED_CHECKING",            EVENT_FINISHED_CHECKING,
+        "EVENT_TORRENT_PAUSED",               EVENT_TORRENT_PAUSED,
         "STATE_QUEUED",                       STATE_QUEUED,
         "STATE_CHECKING",                     STATE_CHECKING,
         "STATE_CONNECTING",                   STATE_CONNECTING,
@@ -449,6 +552,13 @@ static PyObject *torrent_init(PyObject *self, PyObject *args)
 
 static PyObject *torrent_quit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_quit()" << std::endl;
+#endif
     printf("core: removing torrents...\r\n");
     delete M_torrents;
     printf("core: removing settings...\r\n");
@@ -464,6 +574,13 @@ static PyObject *torrent_quit(PyObject *self, PyObject *args)
 
 static PyObject *torrent_save_fastresume(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_save_fastresume()" << std::endl;
+#endif
     python_long unique_ID;
     const char *torrent_name;
     if (!PyArg_ParseTuple(args, "is", &unique_ID, &torrent_name))
@@ -495,6 +612,13 @@ static PyObject *torrent_save_fastresume(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_max_half_open(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_max_half_open()" << std::endl;
+#endif
     python_long arg;
     if (!PyArg_ParseTuple(args, "i", &arg))
         return NULL;
@@ -532,6 +656,13 @@ static PyObject *torrent_set_max_half_open(PyObject *self, PyObject *args)
 
 static PyObject *torrent_connection_speed(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_connection_speed()" << std::endl;
+#endif
     python_long arg;
     if (!PyArg_ParseTuple(args, "i", &arg))
         return NULL;
@@ -543,6 +674,13 @@ static PyObject *torrent_connection_speed(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_download_rate_limit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_download_rate_limit()" << std::endl;
+#endif
     python_long arg;
     if (!PyArg_ParseTuple(args, "i", &arg))
         return NULL;
@@ -554,6 +692,13 @@ static PyObject *torrent_set_download_rate_limit(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_upload_rate_limit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_upload_rate_limit()" << std::endl;
+#endif
     python_long arg;
     if (!PyArg_ParseTuple(args, "i", &arg))
         return NULL;
@@ -564,6 +709,13 @@ static PyObject *torrent_set_upload_rate_limit(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_per_upload_rate_limit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_per_upload_rate_limit()" << std::endl;
+#endif
     python_long unique_ID, speed;
     if (!PyArg_ParseTuple(args, "ii", &unique_ID, &speed))
         return NULL;
@@ -581,6 +733,13 @@ static PyObject *torrent_set_per_upload_rate_limit(PyObject *self, PyObject *arg
 
 static PyObject *torrent_get_per_upload_rate_limit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_per_upload_rate_limit()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -598,6 +757,13 @@ static PyObject *torrent_get_per_upload_rate_limit(PyObject *self, PyObject *arg
 
 static PyObject *torrent_set_per_download_rate_limit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_per_download_rate_limit()" << std::endl;
+#endif
     python_long unique_ID, speed;
 
     if (!PyArg_ParseTuple(args, "ii", &unique_ID, &speed))
@@ -615,6 +781,13 @@ static PyObject *torrent_set_per_download_rate_limit(PyObject *self, PyObject *a
 
 static PyObject *torrent_get_per_download_rate_limit(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_per_download_rate_limit()" << std::endl;
+#endif
     python_long unique_ID;
 
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
@@ -631,6 +804,13 @@ static PyObject *torrent_get_per_download_rate_limit(PyObject *self, PyObject *a
 
 static PyObject *torrent_set_listen_on(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_listen_on()" << std::endl;
+#endif
     PyObject *port_vec;
     if (!PyArg_ParseTuple(args, "O", &port_vec))
         return NULL;
@@ -650,6 +830,13 @@ static PyObject *torrent_set_listen_on(PyObject *self, PyObject *args)
 
 static PyObject *torrent_is_listening(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_is_listening()" << std::endl;
+#endif
     long ret = (M_ses->is_listening() != 0);
 
     return Py_BuildValue("i", ret);
@@ -658,12 +845,26 @@ static PyObject *torrent_is_listening(PyObject *self, PyObject *args)
 
 static PyObject *torrent_listening_port(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_listening_port()" << std::endl;
+#endif
     return Py_BuildValue("i", (python_long)M_ses->listen_port());
 }
 
 
 static PyObject *torrent_set_max_upload_slots_global(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_max_upload_slots_global()" << std::endl;
+#endif
     python_long max_up;
     if (!PyArg_ParseTuple(args, "i", &max_up))
         return NULL;
@@ -676,6 +877,13 @@ static PyObject *torrent_set_max_upload_slots_global(PyObject *self, PyObject *a
 
 static PyObject *torrent_set_max_connections_global(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_max_connections_global()" << std::endl;
+#endif
     python_long max_conn;
     if (!PyArg_ParseTuple(args, "i", &max_conn))
         return NULL;
@@ -687,6 +895,13 @@ static PyObject *torrent_set_max_connections_global(PyObject *self, PyObject *ar
 
 static PyObject *torrent_dump_file_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_dump_file_info()" << std::endl;
+#endif
     const char *name;
     if (!PyArg_ParseTuple(args, "s", &name))
         return NULL;
@@ -721,6 +936,13 @@ static PyObject *torrent_dump_file_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_dump_trackers(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_dump_trackers()" << std::endl;
+#endif
     const char *name;
     if (!PyArg_ParseTuple(args, "s", &name))
         return NULL;
@@ -739,6 +961,13 @@ static PyObject *torrent_dump_trackers(PyObject *self, PyObject *args)
 
 static PyObject *torrent_add_torrent(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_add_torrent()" << std::endl;
+#endif
     const char *name, *save_dir;
     python_long compact;
     if (!PyArg_ParseTuple(args, "ssi", &name, &save_dir, &compact))
@@ -747,7 +976,7 @@ static PyObject *torrent_add_torrent(PyObject *self, PyObject *args)
     boost::filesystem::path save_dir_2  (save_dir, empty_name_check);
     try
     {
-        long ret = internal_add_torrent(name, 1.0, compact, save_dir_2);
+        long ret = internal_add_torrent(name, compact, save_dir_2);
         if (PyErr_Occurred())
             return NULL;
         else
@@ -767,6 +996,13 @@ static PyObject *torrent_add_torrent(PyObject *self, PyObject *args)
 
 static PyObject *torrent_test_duplicate(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_test_duplicate()" << std::endl;
+#endif
     const char *name;
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "si", &name, &unique_ID))
@@ -783,6 +1019,13 @@ static PyObject *torrent_test_duplicate(PyObject *self, PyObject *args)
 
 static PyObject *torrent_move_storage(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_move_storage()" << std::endl;
+#endif
     const char *move_dir;
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "is", &unique_ID, &move_dir))
@@ -800,6 +1043,13 @@ static PyObject *torrent_move_storage(PyObject *self, PyObject *args)
 
 static PyObject *torrent_remove_torrent(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_remove_torrent()" << std::endl;
+#endif
     python_long unique_ID, delete_files;
     if (!PyArg_ParseTuple(args, "ii", &unique_ID, &delete_files))
         return NULL;
@@ -816,12 +1066,26 @@ static PyObject *torrent_remove_torrent(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_num_torrents(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_num_torrents()" << std::endl;
+#endif
     return Py_BuildValue("i", M_torrents->size());
 }
 
 
 static PyObject *torrent_reannounce(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_reannounce()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -838,6 +1102,13 @@ static PyObject *torrent_reannounce(PyObject *self, PyObject *args)
 
 static PyObject *torrent_pause(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_pause()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -863,6 +1134,13 @@ static PyObject *torrent_pause(PyObject *self, PyObject *args)
 
 static PyObject *torrent_resume(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_resume()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -887,6 +1165,13 @@ static PyObject *torrent_resume(PyObject *self, PyObject *args)
 
 static PyObject *torrent_has_piece(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_has_piece()" << std::endl;
+#endif
     python_long unique_ID;
     long piece_index;
     bool has_piece;
@@ -906,6 +1191,13 @@ static PyObject *torrent_has_piece(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_all_piece_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_all_piece_info()" << std::endl;
+#endif
    python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -937,6 +1229,13 @@ static PyObject *torrent_get_all_piece_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_piece_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_piece_info()" << std::endl;
+#endif
     python_long unique_ID;
     long piece_index;
     if (!PyArg_ParseTuple(args, "ii", &unique_ID, &piece_index))
@@ -961,6 +1260,13 @@ static PyObject *torrent_get_piece_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_torrent_state(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_torrent_state()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -1009,7 +1315,7 @@ static PyObject *torrent_get_torrent_state(PyObject *self, PyObject *args)
         PyTuple_SetItem(pieces_range, i/2, rangepos);   
     }
 
-    PyObject *ret = Py_BuildValue("{s:s,s:i,s:i,s:l,s:l,s:f,s:f,s:b,s:f,s:L,s:L,s:s,s:s,s:f,s:L,s:L,s:O,s:i,s:i,s:L,s:L,s:i,s:l,s:l,s:b,s:b,s:L,s:L,s:L}",
+    PyObject *ret = Py_BuildValue("{s:s,s:i,s:i,s:l,s:l,s:f,s:f,s:b,s:f,s:L,s:L,s:s,s:s,s:f,s:L,s:L,s:O,s:i,s:i,s:L,s:i,s:i,s:l,s:l,s:b,s:b,s:L,s:L,s:L,s:b}",
         "name",               t.handle.get_torrent_info().name().c_str(),
         "num_files",          t.handle.get_torrent_info().num_files(),
         "state",              s.state,
@@ -1038,7 +1344,8 @@ static PyObject *torrent_get_torrent_state(PyObject *self, PyObject *args)
         "is_seed",            t.handle.is_seed(),
         "total_done",          s.total_done,
         "total_wanted",       s.total_wanted,
-        "total_wanted_done",  s.total_wanted_done);
+        "total_wanted_done",  s.total_wanted_done,
+        "is_finished",            t.handle.is_finished());
         
         Py_DECREF(pieces_range);
         return ret;
@@ -1046,6 +1353,13 @@ static PyObject *torrent_get_torrent_state(PyObject *self, PyObject *args)
 
 static PyObject *torrent_pop_event(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_pop_event()" << std::endl;
+#endif
     std::auto_ptr<alert> a;
 
     a = M_ses->pop_alert();
@@ -1271,10 +1585,11 @@ static PyObject *torrent_pop_event(PyObject *self, PyObject *args)
             return NULL;
 
         if (handle_exists(handle))
-            return Py_BuildValue("{s:i,s:i,s:s}",
+            return Py_BuildValue("{s:i,s:i,s:i,s:i,s:s}",
                 "event_type",       EVENT_TRACKER_REPLY,
-                "unique_ID",
-                M_torrents->at(index).unique_ID,
+                "unique_ID",        M_torrents->at(index).unique_ID,
+                "num_complete",     handle.status().num_complete,
+                "num_incomplete",   handle.status().num_incomplete,
                 "message",          a->msg().c_str());
         else
             { Py_INCREF(Py_None); return Py_None; }
@@ -1293,8 +1608,37 @@ static PyObject *torrent_pop_event(PyObject *self, PyObject *args)
                 "message",          a->msg().c_str());
         else
             { Py_INCREF(Py_None); return Py_None; }
-    }
+    } else if (dynamic_cast<torrent_checked_alert*>(popped_alert))
+    {
+        torrent_handle handle = (dynamic_cast<torrent_checked_alert*>(popped_alert))->handle;
 
+        long index = get_torrent_index(handle);
+        if (PyErr_Occurred())
+            return NULL;
+
+        if (handle_exists(handle))
+            return Py_BuildValue("{s:i,s:i,s:s}", 
+                "event_type", EVENT_FINISHED_CHECKING,
+                "unique_ID", M_torrents->at(index).unique_ID,
+                "message", a->msg().c_str());
+        else
+            { Py_INCREF(Py_None); return Py_None; }
+    } else if (dynamic_cast<torrent_paused_alert*>(popped_alert))
+    {
+        torrent_handle handle = (dynamic_cast<torrent_paused_alert*>(popped_alert))->handle;
+
+        long index = get_torrent_index(handle);
+        if (PyErr_Occurred())
+            return NULL;
+
+        if (handle_exists(handle))
+            return Py_BuildValue("{s:i,s:i,s:s}", 
+                "event_type", EVENT_TORRENT_PAUSED,
+                "unique_ID", M_torrents->at(index).unique_ID,
+                "message", a->msg().c_str());
+        else
+            { Py_INCREF(Py_None); return Py_None; }
+    }
     return Py_BuildValue("{s:i,s:s}", "event_type", EVENT_OTHER,
         "message",   a->msg().c_str()     );
 }
@@ -1302,6 +1646,13 @@ static PyObject *torrent_pop_event(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_session_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_session_info()" << std::endl;
+#endif
     session_status s = M_ses->status();
 
     return Py_BuildValue("{s:l,s:f,s:f,s:ls:l,s:f,s:f}",
@@ -1317,6 +1668,13 @@ static PyObject *torrent_get_session_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_peer_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_peer_info()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -1389,6 +1747,13 @@ static PyObject *torrent_get_peer_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_file_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_file_info()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -1439,6 +1804,13 @@ static PyObject *torrent_get_file_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_file_piece_range(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_file_piece_range()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -1483,12 +1855,26 @@ static PyObject *torrent_get_file_piece_range(PyObject *self, PyObject *args)
 
 static PyObject *torrent_constants(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_constants()" << std::endl;
+#endif
     Py_INCREF(M_constants); return M_constants;
 }
 
 
 static PyObject *torrent_start_DHT(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_start_DHT()" << std::endl;
+#endif
     const char *DHT_path;
     if (!PyArg_ParseTuple(args, "s", &DHT_path))
         return NULL;
@@ -1530,6 +1916,13 @@ static PyObject *torrent_start_DHT(PyObject *self, PyObject *args)
 
 static PyObject *torrent_stop_DHT(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_stop_DHT()" << std::endl;
+#endif
     const char *DHT_path;
     if (!PyArg_ParseTuple(args, "s", &DHT_path))
         return NULL;
@@ -1562,6 +1955,13 @@ static PyObject *torrent_stop_DHT(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_DHT_info(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_DHT_info()" << std::endl;
+#endif
     entry DHT_state = M_ses->dht_state();
 
     return Py_BuildValue("l", python_long(count_DHT_peers(DHT_state)));
@@ -1570,6 +1970,13 @@ static PyObject *torrent_get_DHT_info(PyObject *self, PyObject *args)
 
 static PyObject *torrent_create_torrent(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_create_torrent()" << std::endl;
+#endif
     using namespace libtorrent;
     using namespace boost::filesystem;
 
@@ -1650,6 +2057,13 @@ static PyObject *torrent_create_torrent(PyObject *self, PyObject *args)
 
 static PyObject *torrent_reset_IP_filter(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_reset_IP_filter()" << std::endl;
+#endif
     // Remove existing filter, if there is one
     if (M_the_filter != NULL)
         delete M_the_filter;
@@ -1664,6 +2078,13 @@ static PyObject *torrent_reset_IP_filter(PyObject *self, PyObject *args)
 
 static PyObject *torrent_add_range_to_IP_filter(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_add_range_to_IP_filter()" << std::endl;
+#endif
     if (M_the_filter == NULL) {
         RAISE_PTR(DelugeError, "No filter defined, use reset_IP_filter");
     }
@@ -1681,6 +2102,13 @@ static PyObject *torrent_add_range_to_IP_filter(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_IP_filter(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_IP_filter()" << std::endl;
+#endif
     if (M_the_filter == NULL) {
         RAISE_PTR(DelugeError, "No filter defined, use reset_IP_filter");
     }
@@ -1692,6 +2120,13 @@ static PyObject *torrent_set_IP_filter(PyObject *self, PyObject *args)
 
 static PyObject *torrent_use_upnp(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_use_upnp()" << std::endl;
+#endif
     python_long action;
     PyArg_ParseTuple(args, "i", &action);
 
@@ -1708,6 +2143,13 @@ static PyObject *torrent_use_upnp(PyObject *self, PyObject *args)
 
 static PyObject *torrent_use_lsd(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_use_lsd()" << std::endl;
+#endif
     python_long action;
     PyArg_ParseTuple(args, "i", &action);
 
@@ -1722,6 +2164,13 @@ static PyObject *torrent_use_lsd(PyObject *self, PyObject *args)
 
 static PyObject *torrent_use_natpmp(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_use_natpmp()" << std::endl;
+#endif
     python_long action;
 
     PyArg_ParseTuple(args, "i", &action);
@@ -1738,6 +2187,13 @@ static PyObject *torrent_use_natpmp(PyObject *self, PyObject *args)
 
 static PyObject *torrent_use_utpex(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_use_utpex()" << std::endl;
+#endif
     python_long action;
 
     PyArg_ParseTuple(args, "i", &action);
@@ -1751,6 +2207,13 @@ static PyObject *torrent_use_utpex(PyObject *self, PyObject *args)
 
 static PyObject *torrent_pe_settings(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_pe_settings()" << std::endl;
+#endif
     M_pe_settings = new pe_settings();
     libtorrent::pe_settings::enc_policy out, in, prefer;
     libtorrent::pe_settings::enc_level    level;
@@ -1769,6 +2232,13 @@ static PyObject *torrent_pe_settings(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_ratio(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_ratio()" << std::endl;
+#endif
     python_long unique_ID;
     float num;
     if (!PyArg_ParseTuple(args, "if", &unique_ID, &num))
@@ -1785,6 +2255,13 @@ static PyObject *torrent_set_ratio(PyObject *self, PyObject *args)
 
 static PyObject *torrent_proxy_settings(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_proxy_settings()" << std::endl;
+#endif
     char *server, *login, *pasw;
     int portnum;
     libtorrent::proxy_settings::proxy_type proxytype;
@@ -1820,6 +2297,13 @@ static PyObject *torrent_proxy_settings(PyObject *self, PyObject *args)
 
 static PyObject *torrent_get_trackers(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_get_trackers()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
@@ -1851,6 +2335,13 @@ static PyObject *torrent_get_trackers(PyObject *self, PyObject *args)
 
 static PyObject *torrent_replace_trackers(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_replace_trackers()" << std::endl;
+#endif
   python_long unique_ID;
   const char* tracker;
   if (!PyArg_ParseTuple(args, "iz", &unique_ID, &tracker))
@@ -1887,6 +2378,13 @@ static PyObject *torrent_replace_trackers(PyObject *self, PyObject *args)
 }
 static PyObject *torrent_prioritize_files(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_prioritize_files()" << std::endl;
+#endif
     python_long unique_ID;
     PyObject *priorities_list_object;
     
@@ -1916,6 +2414,13 @@ static PyObject *torrent_prioritize_first_last_pieces(PyObject *self,
                                                       PyObject *args)
 {
 #define FIRST_LAST_PRIO 6
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_prioritize_first_last_pieces()" << std::endl;
+#endif
     
     // Prioritize first and last 1% of file size bytes in each file of torrent
     // with unique_ID
@@ -1984,6 +2489,13 @@ static PyObject *torrent_prioritize_first_last_pieces(PyObject *self,
 
 static PyObject *torrent_set_priv(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_priv()" << std::endl;
+#endif
     using namespace libtorrent;
     python_long unique_ID;
     bool onoff;
@@ -2004,6 +2516,13 @@ static PyObject *torrent_set_priv(PyObject *self, PyObject *args)
 
 static PyObject *torrent_set_max_connections_per_torrent(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_max_connections_per_torrent()" << std::endl;
+#endif
     python_long unique_ID, max_connections;
     
     if (!PyArg_ParseTuple(args, "ii", &unique_ID, &max_connections))
@@ -2020,6 +2539,13 @@ static PyObject *torrent_set_max_connections_per_torrent(PyObject *self, PyObjec
 
 static PyObject *torrent_set_max_upload_slots_per_torrent(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_set_max_upload_slots_per_torrent()" << std::endl;
+#endif
     python_long unique_ID, max_upload_slots;
     
     if (!PyArg_ParseTuple(args, "ii", &unique_ID, &max_upload_slots))
@@ -2036,6 +2562,13 @@ static PyObject *torrent_set_max_upload_slots_per_torrent(PyObject *self, PyObje
 
 static PyObject *torrent_add_url_seed(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_add_url_seed()" << std::endl;
+#endif
     python_long unique_ID;
     char *address;
     
@@ -2052,6 +2585,13 @@ static PyObject *torrent_add_url_seed(PyObject *self, PyObject *args)
 
 static PyObject *torrent_scrape_tracker(PyObject *self, PyObject *args)
 {
+#ifdef DELUGE_CORE_DEBUG
+time_t rawtime;
+struct tm *timeinfo;
+time(&rawtime);
+timeinfo = localtime(&rawtime);
+std::cout << asctime(timeinfo) << " torrent_scrape_tracker()" << std::endl;
+#endif
     python_long unique_ID;
     if (!PyArg_ParseTuple(args, "i", &unique_ID))
         return NULL;
