@@ -118,7 +118,7 @@ class DelugeGTK:
             import time
             
             def _run_script():
-                common.send_info()
+                common.send_info(self.config.get("enabled_plugins"))
             
             info_file = os.path.join(common.CONFIG_DIR, 'infosent')
             
@@ -156,13 +156,9 @@ class DelugeGTK:
 
         self.dht_timer = 0
         self.dht_skip = False
-        self.memory_timer = 0
         for torrent in self.manager.get_queue():
             unique_ID = self.manager.get_torrent_unique_id(torrent)
             try:
-                if self.manager.unique_IDs[unique_ID].uploaded_memory:
-                    self.manager.unique_IDs[unique_ID].initial_uploaded_memory = \
-                        self.manager.unique_IDs[unique_ID].uploaded_memory
                 if self.manager.unique_IDs[unique_ID].trackers_changed:
                     self.manager.replace_trackers(unique_ID, \
                         self.manager.unique_IDs[unique_ID].trackers)
@@ -565,7 +561,7 @@ window, please enter your password"))
                         "remove_torrent": self.remove_torrent_clicked,
                         "edit_trackers": self.list_of_trackers,
                         "tor_start": self.tor_start,
-                        "torrent_recheck": self.torrent_recheck,
+                        "torrent_switch_recheck": self.torrent_switch_recheck,
                         "tor_pause": self.tor_pause,
                         "update_tracker": self.update_tracker,
                         "clear_finished": self.clear_finished,
@@ -733,22 +729,6 @@ window, please enter your password"))
         self.open_folder(view_column)
 
     def open_folder(self, widget, uids=None):
-        if not common.windows_check():
-            if self.config.get("open_folder_stock"):
-                if self.config.get("file_manager") == common.FileManager.xdg:
-                    file_manager = "xdg-open"
-                elif self.config.get("file_manager") == common.FileManager.\
-                    konqueror:
-                    file_manager = "konqueror"
-                elif self.config.get("file_manager") == common.FileManager.nautilus:
-                    file_manager = "nautilus"
-                elif self.config.get("file_manager") == common.FileManager.thunar:
-                    file_manager = "thunar"
-            else:
-                file_manager = self.config.get("open_folder_location")
-        else:
-            file_manager = "explorer.exe"
-
         if not uids:
             unique_ids = self.get_selected_torrent_rows()
         else:
@@ -757,71 +737,88 @@ window, please enter your password"))
         try:
             for uid in unique_ids:
                 torrent_path = self.manager.get_torrent_path(uid)
-                common.exec_command(file_manager, torrent_path)
+                if not common.windows_check():
+                    file_manager = "xdg-open"
+                    common.exec_command(file_manager, torrent_path)
+                else:
+                    common.exec_command(executable=None, parameters=torrent_path)
+                    
         except KeyError:
             pass
 
-    def torrent_recheck(self, widget):
+    def torrent_switch_recheck(self, widget=None, switch=False):
         unique_ids = self.get_selected_torrent_rows()
         for uid in unique_ids:
             torrent_state = self.manager.get_torrent_state(uid)
             order = torrent_state['queue_pos']
             path = self.manager.unique_IDs[uid].filename
-            save_dir = self.manager.unique_IDs[uid].save_dir
-            trackerslist = self.manager.unique_IDs[uid].trackers
-            try:
-                trackers_changed = self.manager.unique_IDs[uid].trackers_changed
-            except AttributeError:
-                 trackers_changed = 0
-            self.manager.save_upmem()
-            uploaded_memory = self.manager.unique_IDs[uid].uploaded_memory
-            priorities = self.manager.get_priorities(uid)
-            save_info = [path, save_dir, order, trackerslist, \
-                            uploaded_memory, priorities, trackers_changed]
-            try:
-                os.remove(self.manager.unique_IDs[uid].filename + ".fastresume")
-            except:
-                pass
-            self.manager.remove_torrent(uid, False, False)
-            self.torrent_model_remove(uid)
-            self.update()
-            unique_ID = self.manager.add_torrent(save_info[0], save_info[1], \
-                self.config.get("use_compact_storage"))
-            self.torrent_model_append(unique_ID)
-            self.update()
-            self.manager.prioritize_files(unique_ID, save_info[5], update_files_removed=False)
-            if save_info[4]:
-                self.manager.unique_IDs[unique_ID].initial_uploaded_memory = \
-                    save_info[4]
-                self.manager.save_upmem()
-            if save_info[6]:
-                try:
-                    self.manager.replace_trackers(unique_ID, save_info[3])
-                except:
-                   pass
-            torrent_state = self.manager.get_torrent_state(unique_ID)
-            current_order = torrent_state['queue_pos']
-            if current_order > save_info[2]:
-                diff = current_order - save_info[2]
-                for x in range(diff):
-                    self.manager.queue_up(unique_ID)
-                self.update()
+            if not switch:
+                save_dir = self.manager.unique_IDs[uid].save_dir
+                delete_old = False
             else:
-                diff = save_info[2] - current_order
-                for x in range(diff):
-                    self.manager.queue_down(unique_ID)
+                save_dir = dialogs.show_directory_chooser_dialog(self.window, \
+                _("Choose a directory to switch torrent source to" + " - %s" % \
+                    self.manager.get_torrent_state(uid)['name']))
+                if save_dir:
+                    delete_old = dialogs.show_popup_question(self.window, \
+                        _("Delete the old torrent source?"))
+                else:
+                    delete_old = False
+            if save_dir:
+                trackerslist = self.manager.unique_IDs[uid].trackers
+                try:
+                    trackers_changed = self.manager.unique_IDs[uid].trackers_changed
+                except AttributeError:
+                     trackers_changed = 0
+                self.manager.pickle_state()
+                uploaded_memory = self.manager.unique_IDs[uid].uploaded_memory
+                priorities = self.manager.get_priorities(uid)
+                save_info = [path, save_dir, order, trackerslist, \
+                                uploaded_memory, priorities, trackers_changed, \
+                            delete_old]
+                try:
+                    os.remove(self.manager.unique_IDs[uid].filename + ".fastresume")
+                except:
+                    pass
+                if save_info[7]:
+                    self.manager.remove_torrent(uid, True, False)
+                else:
+                    self.manager.remove_torrent(uid, False, False)
+                self.torrent_model_remove(uid)
                 self.update()
-
+                unique_ID = self.manager.add_torrent(save_info[0], save_info[1], \
+                    self.config.get("use_compact_storage"))
+                self.torrent_model_append(unique_ID)
+                self.update()
+                self.manager.prioritize_files(unique_ID, save_info[5], update_files_removed=False)
+                if save_info[4]:
+                    self.manager.unique_IDs[unique_ID].uploaded_memory = \
+                        save_info[4]
+                    self.manager.pickle_state()
+                if save_info[6]:
+                    try:
+                        self.manager.replace_trackers(unique_ID, save_info[3])
+                    except:
+                       pass
+                torrent_state = self.manager.get_torrent_state(unique_ID)
+                current_order = torrent_state['queue_pos']
+                if current_order > save_info[2]:
+                    diff = current_order - save_info[2]
+                    for x in range(diff):
+                        self.manager.queue_up(unique_ID)
+                    self.update()
+                else:
+                    diff = save_info[2] - current_order
+                    for x in range(diff):
+                        self.manager.queue_down(unique_ID)
+                    self.update()
+    
     def tor_start(self, widget):
         unique_ids = self.get_selected_torrent_rows()
         try:
             for uid in unique_ids:
                 torrent_state = self.manager.get_torrent_state(uid)
                 if torrent_state["is_paused"]:
-                    try:
-                        os.remove(self.manager.unique_IDs[uid].filename + ".fastresume")
-                    except:
-                        pass
                     self.manager.set_user_pause(uid, False, True)
             
             self.update()
@@ -835,8 +832,6 @@ window, please enter your password"))
             for uid in unique_ids:
                 self.manager.set_user_pause(uid, True, True)
                 torrent_state = self.manager.get_torrent_state(uid)
-                if torrent_state["is_paused"] == 0:
-                    self.manager.save_fastresume_data(uid)
             self.update()
         except:
             pass
@@ -1033,6 +1028,8 @@ window, please enter your password"))
         # focus on startup
         self.update()
         gobject.timeout_add(int(1000*self.manager.config.get("gui_update_interval")), self.update)
+        gobject.timeout_add(250000, self.manager.pickle_state)
+        gobject.timeout_add(300000, self.manager.save_fastresume_data)
 
         # Load plugins after we showed main window (if not started in tray)
         self.load_plugins()
@@ -1046,7 +1043,7 @@ window, please enter your password"))
             gtk.main()
             gtk.gdk.threads_leave()
         except KeyboardInterrupt:
-            self.manager.quit()
+            self.shutdown()
 
     def new_release_check(self):
         import socket
@@ -1113,11 +1110,6 @@ of Deluge.  Would you like to be taken to our download site?"))
         
         self.update_interface = self.window.get_property("visible") and not \
             self.is_minimized
-        self.memory_timer += 1 
-        if (self.memory_timer == 60):
-            self.manager.save_upmem()
-            self.manager.save_fastresume_data()
-            self.memory_timer = 0
         
         # Make sure that the interface still exists
         try:
@@ -1446,19 +1438,19 @@ nice_need + "\n" + _("Available Space:") + " " + nice_free)
             
     def launchpad(self, obj=None):
         common.open_url_in_browser('https://translations.launchpad.net/deluge/\
-trunk/+pots/deluge', self.plugins)
+trunk/+pots/deluge')
 
     def launch_faq(self, obj=None):
-        common.open_url_in_browser('http://deluge-torrent.org/faq.php', self.plugins)
+        common.open_url_in_browser('http://deluge-torrent.org/faq.php')
 
     def launch_donate(self, obj=None):
-        common.open_url_in_browser('http://deluge-torrent.org/downloads.php', self.plugins)
+        common.open_url_in_browser('http://deluge-torrent.org/downloads.php')
 
     def launch_community(self, obj=None):
-        common.open_url_in_browser('http://forum.deluge-torrent.org/', self.plugins)
+        common.open_url_in_browser('http://forum.deluge-torrent.org/')
 
     def launch_homepage(self, obj=None):
-        common.open_url_in_browser('http://deluge-torrent.org/', self.plugins)
+        common.open_url_in_browser('http://deluge-torrent.org/')
             
     def add_torrent_clicked(self, obj=None):
         torrent = dialogs.show_file_open_dialog(self.window)
@@ -1745,7 +1737,7 @@ want to remove all seeding torrents?")):
             else:
                 self.shutdown()
 
-    def shutdown(self, arg=None):
+    def shutdown(self, *arg):
         gtk.quit_add(0, self.manager.quit)
         self.window.hide()
         self.save_column_widths()
